@@ -21,7 +21,10 @@
  *	    Ettore Perazzoli
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <gtk/gtksignal.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkcheckbutton.h>
@@ -39,10 +42,9 @@
 #include <libgnomeprintui/gnome-print-preview.h>
 #include <libgnomeprintui/gnome-print-dialog.h>
 #include <bonobo/bonobo-control.h>
-#include <bonobo/bonobo-property-bag.h>
 #include <bonobo/bonobo-ui-util.h>
-#include <gal/widgets/e-gui-utils.h>
 #include <e-util/e-dialog-utils.h>
+#include <e-util/e-print.h>
 #include "dialogs/cal-prefs-dialog.h"
 #include "calendar-config.h"
 #include "calendar-commands.h"
@@ -51,24 +53,12 @@
 #include "print.h"
 #include "tasks-control.h"
 #include "evolution-shell-component-utils.h"
+#include "e-util/e-menu.h"
+#include "itip-utils.h"
 
-#define TASKS_CONTROL_PROPERTY_URI		"folder_uri"
-#define TASKS_CONTROL_PROPERTY_URI_IDX		1
 #define FIXED_MARGIN                            .05
 
 
-static void tasks_control_properties_init	(BonoboControl		*control,
-						 ETasks			*tasks);
-static void tasks_control_get_property		(BonoboPropertyBag	*bag,
-						 BonoboArg		*arg,
-						 guint			 arg_id,
-						 CORBA_Environment      *ev,
-						 gpointer		 user_data);
-static void tasks_control_set_property		(BonoboPropertyBag	*bag,
-						 const BonoboArg	*arg,
-						 guint			 arg_id,
-						 CORBA_Environment      *ev,
-						 gpointer		 user_data);
 static void tasks_control_activate_cb		(BonoboControl		*control,
 						 gboolean		 activate,
 						 gpointer		 user_data);
@@ -102,11 +92,15 @@ static void tasks_control_print_cmd		(BonoboUIComponent	*uic,
 static void tasks_control_print_preview_cmd	(BonoboUIComponent	*uic,
 						 gpointer		 data,
 						 const char		*path);
+static void tasks_control_assign_cmd           (BonoboUIComponent      *uic,
+                                                gpointer               data,
+                                                const char             *path);
 
+static void tasks_control_forward_cmd          (BonoboUIComponent      *uic,
+                                                gpointer               data,
+                                                const char             *path);
 
-static GnomePrintConfig *print_config = NULL;
-
-
+	  
 BonoboControl *
 tasks_control_new (void)
 {
@@ -125,108 +119,12 @@ tasks_control_new (void)
 		return NULL;
 	}
 
-	tasks_control_properties_init (control, E_TASKS (tasks));
-
 	g_signal_connect (control, "activate", G_CALLBACK (tasks_control_activate_cb), tasks);
 
 	return control;
 }
 
 
-/* Creates the property bag for our new control. */
-static void
-tasks_control_properties_init		(BonoboControl		*control,
-					 ETasks			*tasks)
-					 
-{
-	BonoboPropertyBag *pbag;
-
-	pbag = bonobo_property_bag_new (tasks_control_get_property,
-					tasks_control_set_property, tasks);
-
-	bonobo_property_bag_add (pbag,
-				 TASKS_CONTROL_PROPERTY_URI,
-				 TASKS_CONTROL_PROPERTY_URI_IDX,
-				 BONOBO_ARG_STRING,
-				 NULL,
-				 _("The URI of the tasks folder to display"),
-				 0);
-
-	bonobo_control_set_properties (control, bonobo_object_corba_objref (BONOBO_OBJECT (pbag)), NULL);
-	bonobo_object_unref (BONOBO_OBJECT (pbag));
-}
-
-
-/* Gets a property of our control. FIXME: Finish. */
-static void
-tasks_control_get_property		(BonoboPropertyBag	*bag,
-					 BonoboArg		*arg,
-					 guint			 arg_id,
-					 CORBA_Environment      *ev,
-					 gpointer		 user_data)
-{
-	ETasks *tasks = user_data;
-	const char *uri;
-	ECalModel *model;
-
-	switch (arg_id) {
-
-	case TASKS_CONTROL_PROPERTY_URI_IDX:
-		model = e_calendar_table_get_model (e_tasks_get_calendar_table (tasks));
-		uri = e_cal_get_uri (e_cal_model_get_default_client (model));
-		BONOBO_ARG_SET_STRING (arg, uri);
-		break;
-
-	default:
-		g_warning ("Unhandled arg %d\n", arg_id);
-	}
-}
-
-
-static void
-tasks_control_set_property		(BonoboPropertyBag	*bag,
-					 const BonoboArg	*arg,
-					 guint			 arg_id,
-					 CORBA_Environment      *ev,
-					 gpointer		 user_data)
-{
-	ETasks *tasks = user_data;
-	char *uri;
-	ESource *source;
-	ESourceGroup *group;
-
-	switch (arg_id) {
-
-	case TASKS_CONTROL_PROPERTY_URI_IDX:
-		/* FIXME Remove the old uri? */
-		uri = BONOBO_ARG_GET_STRING (arg);
-
-		group = e_source_group_new ("", uri);
-		source = e_source_new ("", "");
-		e_source_set_group (source, group);
-
-		if (!e_tasks_add_todo_source (tasks, source)) {
-			char *msg;
-
-			msg = g_strdup_printf (_("Could not load the tasks in `%s'"), uri);
-			gnome_error_dialog_parented (
-				msg,
-				GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tasks))));
-			g_free (msg);
-		}
-
-		g_object_unref (source);
-		g_object_unref (group);
-
-		break;
-
-	default:
-		g_warning ("Unhandled arg %d\n", arg_id);
-		break;
-	}
-}
-
-
 static void
 tasks_control_activate_cb		(BonoboControl		*control,
 					 gboolean		 activate,
@@ -250,7 +148,12 @@ tasks_control_sensitize_commands (BonoboControl *control, ETasks *tasks, int n_s
 {
 	BonoboUIComponent *uic;
 	gboolean read_only = TRUE;
+	ECal *ecal;
 	ECalModel *model;
+	ECalendarTable *cal_table;
+	ECalModelComponent *comp_data;
+	icalproperty *prop;
+	gboolean is_assigned = FALSE;
 
 	uic = bonobo_control_get_ui_component (control);
 	g_assert (uic != NULL);
@@ -258,8 +161,20 @@ tasks_control_sensitize_commands (BonoboControl *control, ETasks *tasks, int n_s
 	if (bonobo_ui_component_get_container (uic) == CORBA_OBJECT_NIL)
 		return;
 
-	model = e_calendar_table_get_model (e_tasks_get_calendar_table (tasks));
-	e_cal_is_read_only (e_cal_model_get_default_client (model), &read_only, NULL);
+	cal_table = e_tasks_get_calendar_table (tasks);
+	model = e_calendar_table_get_model (cal_table);
+
+	if (n_selected == 1) {
+		comp_data = e_calendar_table_get_selected_comp (cal_table);
+		prop = icalcomponent_get_first_property (comp_data->icalcomp, ICAL_ATTENDEE_PROPERTY);
+		is_assigned = prop ? TRUE : FALSE;
+	}	
+	
+	ecal = e_cal_model_get_default_client (model);
+
+	if (ecal) 
+		e_cal_is_read_only (ecal, &read_only, NULL);
+		
 
 	bonobo_ui_component_set_prop (uic, "/commands/TasksOpenTask", "sensitive",
 				      n_selected != 1 ? "0" : "1",
@@ -282,6 +197,12 @@ tasks_control_sensitize_commands (BonoboControl *control, ETasks *tasks, int n_s
 	bonobo_ui_component_set_prop (uic, "/commands/TasksPurge", "sensitive",
 				      read_only ? "0" : "1",
 				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/TasksAssign", "sensitive",
+				      (is_assigned || read_only || n_selected != 1) ? "0" : "1",
+				      NULL);
+	bonobo_ui_component_set_prop (uic, "/commands/TasksForward", "sensitive",
+				      n_selected != 1 ? "0" : "1", 
+				      NULL);
 }
 
 /* Callback used when the selection in the table changes */
@@ -306,7 +227,9 @@ static BonoboUIVerb verbs [] = {
 	BONOBO_UI_VERB ("TasksPurge", tasks_control_purge_cmd),
 	BONOBO_UI_VERB ("TasksPrint", tasks_control_print_cmd),
 	BONOBO_UI_VERB ("TasksPrintPreview", tasks_control_print_preview_cmd),
-
+	BONOBO_UI_VERB ("TasksAssign", tasks_control_assign_cmd),
+        BONOBO_UI_VERB ("TasksForward", tasks_control_forward_cmd),
+     
 	BONOBO_UI_VERB_END
 };
 
@@ -343,6 +266,7 @@ tasks_control_activate (BonoboControl *control, ETasks *tasks)
 
 	g_signal_connect (tasks, "selection_changed", G_CALLBACK (selection_changed_cb), control);
 
+	e_menu_activate ((EMenu *)gnome_tasks_get_tasks_menu (tasks), uic, 1);
 	cal_table = e_tasks_get_calendar_table (tasks);
 	etable = e_calendar_table_get_table (cal_table);
 	n_selected = e_table_selected_count (etable);
@@ -367,6 +291,7 @@ tasks_control_deactivate (BonoboControl *control, ETasks *tasks)
 
 	g_assert (uic != NULL);
 
+	e_menu_activate ((EMenu *)gnome_tasks_get_tasks_menu (tasks), uic, 0);
 	e_tasks_set_ui_component (tasks, NULL);
 
 	e_tasks_discard_view_menus (tasks);
@@ -513,7 +438,7 @@ print_tasks (ETasks *tasks, gboolean preview)
 	cal_table = e_tasks_get_calendar_table (tasks);
 	etable = e_calendar_table_get_table (E_CALENDAR_TABLE (cal_table));
 
-	print_table (etable, _("Tasks"), preview);
+	print_table (etable, _("Print Tasks"), _("Tasks"), preview);
 }
 
 /* File/Print callback */
@@ -523,45 +448,10 @@ tasks_control_print_cmd (BonoboUIComponent *uic,
 			 const char *path)
 {
 	ETasks *tasks;
-	GtkWidget *gpd;
-	gboolean preview = FALSE;
-	GnomePrintJob *gpm;
-	ECalendarTable *cal_table;
-	ETable *etable;
 
 	tasks = E_TASKS (data);
 
-	if (!print_config)
-		print_config = gnome_print_config_default ();
-
-	gpm = gnome_print_job_new (print_config);
-
-	gpd = gnome_print_dialog_new (gpm, _("Print Tasks"), GNOME_PRINT_DIALOG_COPIES);
-	gtk_dialog_set_default_response (GTK_DIALOG (gpd), GNOME_PRINT_DIALOG_RESPONSE_PRINT);
-
-	/* Run dialog */
-	switch (gtk_dialog_run (GTK_DIALOG (gpd))) {
-	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
-		break;
-
-	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-		preview = TRUE;
-		break;
-
-	case -1:
-		return;
-
-	default:
-		gtk_widget_destroy (gpd);
-		return;
-	}
-
-	cal_table = e_tasks_get_calendar_table (tasks);
-	etable = e_calendar_table_get_table (E_CALENDAR_TABLE (cal_table));
-
-	gtk_widget_destroy (gpd);
-
-	print_tasks (tasks, preview);
+	print_tasks (tasks, FALSE);
 }
 
 static void
@@ -576,3 +466,39 @@ tasks_control_print_preview_cmd (BonoboUIComponent *uic,
 	print_tasks (tasks, TRUE);
 }
 
+static void
+tasks_control_assign_cmd (BonoboUIComponent *uic,
+                         gpointer data,
+                         const char *path)
+{
+              ETasks *tasks;
+              ECalendarTable *cal_table;
+              ECalModelComponent *comp_data;
+
+              tasks = E_TASKS (data);
+              cal_table = e_tasks_get_calendar_table (tasks);
+              comp_data = e_calendar_table_get_selected_comp (cal_table);
+ 	       if (comp_data)
+              e_calendar_table_open_task (cal_table, comp_data, TRUE);
+}
+
+static void
+tasks_control_forward_cmd (BonoboUIComponent *uic,
+                         gpointer data,
+                          const char *path)
+{
+	        ETasks *tasks;
+               ECalendarTable *cal_table;
+               ECalModelComponent *comp_data;
+	
+               tasks = E_TASKS (data);
+               cal_table = e_tasks_get_calendar_table (tasks);
+               comp_data = e_calendar_table_get_selected_comp (cal_table);
+               if (comp_data) {
+                       ECalComponent *comp;
+                       comp = e_cal_component_new ();
+                       e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (comp_data->icalcomp));
+                       itip_send_comp (E_CAL_COMPONENT_METHOD_PUBLISH, comp, comp_data->client, NULL, NULL);
+                       g_object_unref (comp);
+	       }
+}	       
