@@ -56,9 +56,9 @@
 #include <e-util/e-mktemp.h>
 #include <e-util/e-account-list.h>
 #include <e-util/e-dialog-utils.h>
-#include "widgets/misc/e-error.h"
+#include "e-util/e-error.h"
 
-#include <gal/util/e-util.h>
+#include <e-util/e-util.h>
 
 #include "em-utils.h"
 #include "em-composer-utils.h"
@@ -181,7 +181,7 @@ em_utils_configure_account (GtkWidget *parent)
 {
 	EMAccountEditor *emae;
 
-	emae = em_account_editor_new(NULL, EMAE_DRUID);
+	emae = em_account_editor_new(NULL, EMAE_DRUID, "org.gnome.evolution.mail.config.accountDruid");
 	if (parent != NULL)
 		e_dialog_set_transient_for((GtkWindow *)emae->editor, parent);
 
@@ -300,22 +300,24 @@ em_utils_edit_filters (GtkWidget *parent)
 /* Saving messages... */
 
 static GtkWidget *
-emu_get_save_filesel (GtkWidget *parent, const char *title, const char *name)
+emu_get_save_filesel (GtkWidget *parent, const char *title, const char *name, GtkFileChooserAction action)
 {
 	GtkWidget *filesel;
 	const char *dir;
-	char *realname, *filename, *gdir;
+	char *realname, *gdir;
 	GConfClient *gconf;
 
 #ifdef USE_GTKFILECHOOSER
 	filesel = gtk_file_chooser_dialog_new (title,
 					       NULL,
-					       GTK_FILE_CHOOSER_ACTION_SAVE,
+					       action,
 					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					       GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 					       NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (filesel), GTK_RESPONSE_OK);
 #else
+	char *filename;
+
 	filesel = gtk_file_selection_new (title);
 #endif
 	
@@ -437,12 +439,75 @@ em_utils_save_part(GtkWidget *parent, const char *prompt, CamelMimePart *part)
 		}
 	}
 
-	filesel = emu_get_save_filesel(parent, prompt, name);
+	filesel = emu_get_save_filesel(parent, prompt, name, GTK_FILE_CHOOSER_ACTION_SAVE);
 	camel_object_ref(part);
 	g_signal_connect (filesel, "response", G_CALLBACK (emu_save_part_response), part);
 	gtk_widget_show (filesel);
 }
 
+static void
+emu_save_parts_response (GtkWidget *filesel, int response, GSList *parts)
+{
+        char *path = NULL;
+        GSList *selected;
+        if (response == GTK_RESPONSE_OK) {
+#ifdef USE_GTKFILECHOOSER
+                path = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (filesel));
+#else
+                path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
+#endif
+
+                emu_update_save_path(path);
+
+                for ( selected = parts; selected != NULL; selected = selected->next) {
+                        const char *file_name;
+			char *safe_name = NULL;
+                        char *file_path;
+                        CamelMimePart *part = selected->data;
+
+                        file_name = camel_mime_part_get_filename(part);
+                        if (file_name == NULL) {
+                                if (CAMEL_IS_MIME_MESSAGE(part)) {
+                                        file_name = camel_mime_message_get_subject((CamelMimeMessage *)part);
+                                        if (file_name == NULL)
+                                                file_name = _("message");
+                                } else {
+                                	file_name = _("attachment");
+                                }
+                        } else {
+				safe_name = g_strdup(file_name);
+				e_filename_make_safe(safe_name);
+				file_name = safe_name;
+			}
+			
+			file_path = g_build_filename (path, file_name, NULL);
+			if (!g_file_test(file_path, (G_FILE_TEST_EXISTS)))
+				mail_save_part(part, file_path, NULL, NULL);
+			else
+				g_warning ("Could not save %s. File already exists", file_path);
+
+			g_free (file_path);
+			g_free (safe_name);
+                }
+
+		g_free (path);
+        }
+	
+	g_slist_free (parts);
+	gtk_widget_destroy((GtkWidget *)filesel);
+}
+
+void
+em_utils_save_parts (GtkWidget *parent, const char *prompt, GSList * parts)
+{
+        GtkWidget *filesel;
+
+        filesel = emu_get_save_filesel (parent, prompt, NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+        g_signal_connect (filesel, "response", G_CALLBACK (emu_save_parts_response), parts);
+        gtk_widget_show (filesel);
+}
+
+
 /**
  * em_utils_save_part_to_file:
  * @parent: parent window
@@ -540,7 +605,7 @@ em_utils_save_messages (GtkWidget *parent, CamelFolder *folder, GPtrArray *uids)
 	g_return_if_fail (CAMEL_IS_FOLDER (folder));
 	g_return_if_fail (uids != NULL);
 
-	filesel = emu_get_save_filesel(parent, _("Save Message..."), NULL);
+	filesel = emu_get_save_filesel(parent, _("Save Message..."), NULL, GTK_FILE_CHOOSER_ACTION_SAVE);
 	camel_object_ref(folder);
 	
 	data = g_malloc(sizeof(struct _save_messages_data));
@@ -639,8 +704,14 @@ tag_editor_response (GtkWidget *dialog, int button, struct ted_t *ted)
 		
 		camel_folder_freeze (folder);
 		for (i = 0; i < uids->len; i++) {
-			for (t = tags; t; t = t->next)
-				camel_folder_set_message_user_tag (folder, uids->pdata[i], t->name, t->value);
+			CamelMessageInfo *mi = camel_folder_get_message_info(folder, uids->pdata[i]);
+
+			if (mi) {
+				for (t = tags; t; t = t->next)
+					camel_message_info_set_user_tag(mi, t->name, t->value);
+
+				camel_message_info_free(mi);
+			}
 		}
 		
 		camel_folder_thaw (folder);
@@ -685,9 +756,12 @@ em_utils_flag_for_followup (GtkWidget *parent, CamelFolder *folder, GPtrArray *u
 		CamelMessageInfo *info;
 		
 		info = camel_folder_get_message_info (folder, uids->pdata[i]);
-		message_tag_followup_append_message (MESSAGE_TAG_FOLLOWUP (editor),
-						     camel_message_info_from (info),
-						     camel_message_info_subject (info));
+		if (info) {
+			message_tag_followup_append_message (MESSAGE_TAG_FOLLOWUP (editor),
+							     camel_message_info_from (info),
+							     camel_message_info_subject (info));
+			camel_message_info_free(info);
+		}
 	}
 	
 	/* special-case... */
@@ -699,8 +773,8 @@ em_utils_flag_for_followup (GtkWidget *parent, CamelFolder *folder, GPtrArray *u
 			const CamelTag *tags = camel_message_info_user_tags(info);
 
 			if (tags)
-				message_tag_editor_set_tag_list (MESSAGE_TAG_EDITOR (editor), tags);
-			camel_folder_free_message_info (folder, info);
+				message_tag_editor_set_tag_list (MESSAGE_TAG_EDITOR (editor), (CamelTag *)tags);
+			camel_message_info_free(info);
 		}
 	}
 	
@@ -729,9 +803,14 @@ em_utils_flag_for_followup_clear (GtkWidget *parent, CamelFolder *folder, GPtrAr
 	
 	camel_folder_freeze (folder);
 	for (i = 0; i < uids->len; i++) {
-		camel_folder_set_message_user_tag (folder, uids->pdata[i], "follow-up", "");
-		camel_folder_set_message_user_tag (folder, uids->pdata[i], "due-by", "");
-		camel_folder_set_message_user_tag (folder, uids->pdata[i], "completed-on", "");
+		CamelMessageInfo *mi = camel_folder_get_message_info(folder, uids->pdata[i]);
+
+		if (mi) {
+			camel_message_info_set_user_tag(mi, "follow-up", NULL);
+			camel_message_info_set_user_tag(mi, "due-by", NULL);
+			camel_message_info_set_user_tag(mi, "completed-on", NULL);
+			camel_message_info_free(mi);
+		}
 	}
 	camel_folder_thaw (folder);
 	
@@ -762,12 +841,14 @@ em_utils_flag_for_followup_completed (GtkWidget *parent, CamelFolder *folder, GP
 	camel_folder_freeze (folder);
 	for (i = 0; i < uids->len; i++) {
 		const char *tag;
-		
-		tag = camel_folder_get_message_user_tag (folder, uids->pdata[i], "follow-up");
-		if (tag == NULL || *tag == '\0')
-			continue;
-		
-		camel_folder_set_message_user_tag (folder, uids->pdata[i], "completed-on", now);
+		CamelMessageInfo *mi = camel_folder_get_message_info(folder, uids->pdata[i]);
+
+		if (mi) {
+			tag = camel_message_info_user_tag(mi, "follow-up");
+			if (tag && tag[0])
+				camel_message_info_set_user_tag(mi, "completed-on", now);
+			camel_message_info_free(mi);
+		}
 	}
 	camel_folder_thaw (folder);
 	
@@ -1790,7 +1871,8 @@ em_utils_in_addressbook(CamelInternetAddress *iaddr)
 	time_t now;
 
 	/* TODO: check all addresses? */
-	if (!camel_internet_address_get(iaddr, 0, NULL, &addr))
+	if (iaddr == NULL
+	    || !camel_internet_address_get(iaddr, 0, NULL, &addr))
 		return FALSE;
 
 	pthread_mutex_lock(&emu_addr_lock);
